@@ -44,22 +44,30 @@ def get_timediff(dataset,
     return dataset
 
 
-def derive_from_datetime(dataset):
+def derive_from_datetime(dataset, specify_columns=None):
     """
     Usage: [arg1]:[pandas dataframe]
     Prerequisite: Type for datetime columns to be defined correctly
     Description: Derives the hour, weekday, year and month from a datetime column
     Returns: Dataframe [with new columns derived from datetime columns]
     """
-    columns = []
-    for column, dtype in dataset.dtypes.items():
-        if 'datetime' in str(dtype):
-            columns.append(column)
+    if specify_columns is None:
+        columns = []
+        for column, dtype in dataset.dtypes.items():
+            if 'datetime' in str(dtype):
+                columns.append(column)
+                dataset['hour_of_' + column] = dataset[column].apply(lambda x: x.hour)
+                dataset['weekday_of_' + column] = dataset[column].apply(lambda x: x.weekday())
+                dataset['year_of_' + column] = dataset[column].apply(lambda x: x.year)
+                dataset['month_of_' + column] = dataset[column].apply(lambda x: x.month)
+        return dataset, columns
+    else:
+        for column in specify_columns:
             dataset['hour_of_' + column] = dataset[column].apply(lambda x: x.hour)
             dataset['weekday_of_' + column] = dataset[column].apply(lambda x: x.weekday())
             dataset['year_of_' + column] = dataset[column].apply(lambda x: x.year)
             dataset['month_of_' + column] = dataset[column].apply(lambda x: x.month)
-    return dataset, columns
+        return dataset, list(specify_columns)
 
 
 def log_transform(dataset, method='yeojohnson', define_continuous_cols=[], ignore_cols=[], categorical_threshold=0.3):
@@ -120,7 +128,7 @@ def drop_single_valued_cols(dataset):
     if len(single_valued_cols) > 0:
         print('Dropping single valued column(s) ' + ','.join(single_valued_cols))
         dataset = dataset.drop(single_valued_cols, axis=1)
-    return dataset
+    return dataset, single_valued_cols
 
 
 def get_ohe_df(dataset,
@@ -233,10 +241,11 @@ def label_encode(dataset,
     Returns: Label Dict , Dataframe
     """
     mode_val = dataset[col].mode()[0]
-    # dataset[col] = dataset[col].apply(lambda x: str(x).strip()).astype(str).fillna(mode_val)
     dataset[col] = dataset[col].fillna(mode_val)
     label_dict = dict(zip(dataset[col].unique(), np.arange(dataset[col].unique().shape[0])))
-    dataset = dataset.replace({col: label_dict})
+    # dataset = dataset.replace({col: label_dict})
+    dataset[col] = dataset[col].astype(str).map(label_dict)
+    dataset[col] = dataset[col].astype('int')
     dataset[col] = dataset[col].astype('int')
     dataset[col] = dataset[col].astype('category')
     return label_dict, dataset
@@ -492,6 +501,25 @@ def multicollinearity_check(corr_df, threshold=0.7):
     return result_set
 
 
+def get_multicollinearity_removals(corr_df, target_variable, threshold=0.7):
+    res = multicollinearity_check(corr_df, threshold=threshold)
+    corr = list(set([helper.get_absolute(item[1]) for item in res]))
+    corr.sort(reverse=True)
+    tgt_corr = corr_df[target_variable].to_dict()
+    remove_list = []
+    for val in corr:
+        for item in res:
+            if helper.get_absolute(item[1]) == val:
+                cols = item[0]
+                if target_variable not in cols:
+                    if len(helper.intersection(cols, remove_list)) == 0:
+                        if cols[0] < cols[1]:
+                            remove_list.append(cols[0])
+                        else:
+                            remove_list.append(cols[1])
+    return remove_list
+
+
 def dataset_summary(dataset,
                     define_continuous_cols=[],
                     define_nominal_cols=[],
@@ -540,10 +568,18 @@ def dataset_summary(dataset,
         dataset_summary[col]['max'] = dataset[col].max()
         if np.abs(scipy.stats.skew(dataset[col])) > 1:
             dataset_summary[col]['Skewed'] = 'Y'
+        else:
+            dataset_summary[col]['Skewed'] = 'N'
     return dataset_summary
 
 
-def split_dataset(dataset, n_splits, proportion=None, shuffle=False):
+def split_dataset(dataset, n_splits, proportion=None, mode=None, shuffle=False):
+    if mode == 'equal':
+        each_proportion = int((1/n_splits)*100)
+        proportion = [each_proportion for i in range(n_splits-1)]
+        final_val = 100 - sum(proportion)
+        proportion.append(final_val)
+        proportion = [val/100 for val in proportion]
     if len(proportion) != n_splits:
         raise exceptions.ParameterError('n_splits should be equal to the number of values in proportion')
     if sum(proportion) != 1:
@@ -556,20 +592,22 @@ def split_dataset(dataset, n_splits, proportion=None, shuffle=False):
 
     df_list = []
     indices_split = []
-    prev = -1
+    prev = 0
     length = len(indices)
     for ctr in range(n_splits):
         max_records = int(np.floor(proportion[ctr] * length))
-        start = prev + 1
+        start = prev
         end = start + max_records
-        curr_split = indices[start:end+1]
+        if ctr == n_splits-1:
+            end = length
+        curr_split = indices[start:end]
         if n_splits-ctr == 1:
             curr_split = indices[start:]
         indices_split.append(curr_split)
         curr_df = dataset.iloc[curr_split]
+        curr_df = curr_df.reset_index(drop=True)
         df_list.append(curr_df)
         prev = end
-
     return df_list, indices_split
 
 
@@ -578,3 +616,19 @@ def Xy_split(dataset, target_feature):
     y = dataset[[target_feature]]
     return X, y
 
+
+def k_fold(dataset, k, shuffle=True):
+    dfs, _ = split_dataset(dataset, n_splits=k, mode='equal', shuffle=shuffle)
+    result = []
+    for i in range(k):
+        train_list = []
+        for j in range(k):
+            if j == i:
+                test_data = dfs[j]
+            else:
+                train_list.append(dfs[j])
+        train_data = pd.concat(train_list)
+        train_data = train_data.reset_index(drop=True)
+        result.append([train_data, test_data])
+    return result
+
